{$A+,B-,D-,E-,F+,G-,I-,L-,N-,O-,P-,Q-,R+,S+,T-,V-,X-}
{$M 16384,0,655360}
UNIT ParData;

{Ŀ
 Die Unit ParData ermglicht den Transfer von Daten ber die parallele       
 Druckerschnittstelle. Um diese Unit einzusetzen, brauchen Sie:              
  - zwei Rechner mit Druckerschnittstelle                                    
  - ein spezielles Parallelkabel (Verdrahtung siehe unten)                   
                                                                             
 Achtung: Bitte stecken Sie das Parallelkabel erst ein, nachdem (!) beide    
          Rechner eingeschaltet sind. Sonst findet das BIOS die Drucker-     
          schnittstelle nicht!                                               
                 Ŀ                          
                  Verdrahtung des Parallelkabels                           
                                           
 Das Kabel ist symmetrisch verdrahtet, Transfers in beide Richtungen sind    
 also mglich. Eine gleichzeitige Benutzung in beide Richtungen ist nicht    
 mglich.                                                                    
                                                                             
Von der Ltseite(!) sieht ein Parallelstecker folgendermaen aus:            
  Ŀ       
    13   12   11   10    9    8    7    6    5    4    3    2    1         
  Ŀ   25   24   23   22   21   20   19   18   17   16   15   14         
             
Die Nummern bezeichnen die Nummern der entsprechenden Pins auf dem Stecker.  
Oft sind die Nummern auch auf dem Stecker aufgedruckt - das vereinfacht die  
Sache erheblich.                                                             
Um diese Unit zu verwenden, mssen Sie die Leitungen folgendermaen ver-     
binden:                                                                      
Ŀ    
 1 <-->  1     6 <--> 13    11 <-->  9    16 <-->  3    21 <--> 21     
 2 <--> 14     7 <--> 12    12 <-->  7    17 <-->  4    22 <--> 22     
 3 <--> 16     8 <--> 10    13 <-->  6    18 <--> 18    23 <--> 23     
 4 <--> 17     9 <--> 11    14 <-->  2    19 <--> 19    24 <--> 24     
 5 <--> 15    10 <-->  8    15 <-->  5    20 <--> 20    25 <--> 25     
    
Sie werden vermutlich dieses Kabel nicht im Geschft kaufen knnen, da       
eine solche Verdrahtung berhaupt keinem Standard entspricht. Die ein-       
fachste Mglichkeit ist, ein Kabel zu kaufen und entsprechend umzulten.     
Sollte Sie dies ihre handwerklichen Fhigkeiten berschreiten, fragen Sie    
doch einfach einen Computerhndler (z.B. den, bei dem Sie den Rechner        
gekauft haben). Vielleicht ist er ja bereit, ein Kabel fr Sie anzufer-      
tigen.                                                                       
Das Kabel knnen Sie mit dem Programm KabelTst berprfen, das auf der       
Diskette beigelegt ist. Sollten Sie berhaupt keine Reaktion feststellen,    
haben Sie vermutlich das Kabel vor dem Einschalten bereits eingesteckt.      
Dann also nix wie raus mit dem Kabel und neu booten. Die Strobe-Leitung      
ist aktiv, wenn mindestens einer der beiden Computer diese aktiviert.        
}

INTERFACE

USES Dos,Crt;

TYPE tState = (SNotThere,SFree,SWantSend);

PROCEDURE UnitInit(LptNr : BYTE);
  { Initialisiert die parallele Schnittstelle }
  { LPT1 = 1, LPT2 = 2, ...                   }

FUNCTION CheckOther : tState;
  { berprft den Status der anderen Seite. Bei NotThere wurde keine
    Reaktion gemessen, bei SWantSend mchte die andere Seite unbedingt
    senden. Bei SFree kann der eigene Rechner senden.                  }

{ Die Sende- und Empfangsroutinen sind aus Geschwindigkeitsgrnden in drei
  Einzelroutinen aufgespalten. Mit den Startroutinen (StartSend und
  StartReceive) wird die Synchronisation hergestellt. Die eigentliche ber-
  tragung geschieht mit SendBlock bzw. ReceiveBlock, die neben der Puffer-
  adresse die Gre des Blocks enthalten. Ist der Empfngerblock zu klein,
  wird das Programm abgebrochen. StopSend und StopReceive geben den Kanal
  nach der bertragung wieder frei. Sie knnen (und sollten) mehrere Block-
  transfers zwischen den Synchronisationsprozeduren durchfhren.            }

PROCEDURE StartSend;
 { Synchronisation mit Empfnger herstellen }
PROCEDURE SendBlock(VAR Buf; Size : WORD);
 { Die eigentliche Kernroutine des Sendens. }
PROCEDURE StopSend;
 { Datenbertragung abschlieen }

PROCEDURE StartReceive;
 { Synchronisation mit Sender herstellen }
FUNCTION ReceiveBlock(VAR Buf; Size : WORD) : WORD;
 { Die eigentliche Kernroutine des Empfangens }
PROCEDURE StopReceive;
 { Datenbertragung abschlieen }

IMPLEMENTATION

CONST DoStrobe   = 5;    Free     = 0;
      NotStrobe  = 4;    Closed   = 1;

      NULL1     = $10;   ACK1     = $D0;   Break = $60;
      NULL2     = $20;   ACK2     = $48;   ACKError = $78;

CONST nullcode : BYTE = NULL1;
      nullback : BYTE = NULL2;

VAR OwnState              : BYTE;
    daten, status, steuer : WORD;
    OldInt08ptr, OldExit  : POINTER;
    OldInt08              : PROCEDURE ABSOLUTE OldInt08ptr;

PROCEDURE ColdError(st : STRING);
BEGIN
  WriteLn(st);
  RunError;
END;

FUNCTION empfang : BYTE; ASSEMBLER;
  ASM
    MOV   DX,status;
    IN    AL,DX;
    MOV   AH,AL;
    AND   AH,$F8;
    INC   DX;
    IN    AL,DX;
    SHR   AL,1;
    AND   AL,$7;
    OR    AL,AH;
    XOR   AL,$85;
  END;

PROCEDURE swait(code : BYTE); ASSEMBLER;
  ASM
    MOV   CL, code;
    MOV   DX, steuer
  @Loop:
    IN    AL,DX;
    AND   AL, 1;
    CMP   AL,CL
    JNZ   @Loop
  END;

PROCEDURE NewInt08; ASSEMBLER;
ASM
  PUSH   DS
  PUSH   AX
  MOV    AX, Seg @data
  MOV    DS,AX
  CMP    OwnState,Free
  JNZ    @nothing
  PUSH   DX
  MOV    AL,nullback
  XCHG   AL,nullcode
  MOV    nullback,AL
  MOV    DX, daten
  OUT    DX,AL
  POP    DX
@nothing:
  PUSHF
  CALL   OldInt08
  POP    AX
  POP    DS
  IRET
END;

PROCEDURE UnitExit; FAR;
BEGIN
  SetIntVec($08,OldInt08ptr);
  ExitProc := OldExit;
END;

PROCEDURE UnitInit(LptNr : BYTE);
  { Initialisiert die parallele Schnittstelle }
  { LPT1 = 1, LPT2 = 2, ...                   }
BEGIN
  { Adresse aus dem ROM-BIOS-Speicherbereich holen }
  daten  := MemW[$40:$08+2*(LptNr-1)];
  status := daten+1;
  steuer := daten+2;
  { Ports auf Lesen vorbereiten }
  Port[steuer] := NotStrobe; { notwendig, um Daten zu lesen }
  Port[status] := $0;        {            "                 }
  Port[daten ] := NULL1;     { irgendwas mu ja rein...     }
  { Interrupt-Handler installieren ... }
  OwnState := Free;
  GetIntVec($08,OldInt08ptr);
  OldExit := ExitProc;
  ExitProc := @UnitExit;
  SetIntVec($08,@NewInt08);
END;

FUNCTION CheckOther : tState;
VAR old, new, i : BYTE;
BEGIN
  i := 0; old := NULL2;
  REPEAT
    Delay(2); Inc(i);
    new := empfang;
    IF new <> old THEN BEGIN
      IF (old = NULL1) AND (new=NULL2) THEN BEGIN
        IF Odd(Port[steuer]) THEN CheckOther := SWantSend
                             ELSE CheckOther := SFree;
        Exit;
      END;
      old := new;
    END;
  UNTIL (i >= 100);
  CheckOther := SNotThere;
END; { CheckOtherState }

PROCEDURE StartSend;
 { Synchronisation mit Empfnger herstellen }
BEGIN
  Port[steuer] := DoStrobe;   { Sendebereitschaft mitteilen ... }
  { ------- Hier schaltet sich der SyncToReceive ein... ------- }
  WHILE empfang <> ACK1 DO;   { Antwort abwarten ...            }
  OwnState := Closed;         { Timer abschalten ...            }
  Port[steuer] := NotStrobe;  { an Empfnger melden ...         }
  WHILE empfang <> ACK2 DO;   { und besttigen lassen ...       }
END;

PROCEDURE StopSend;
BEGIN
  { Transfer beenden }
  Port[daten]  := NULL2;
  Port[steuer] := NotStrobe;
  OwnState     := Free;
END;

PROCEDURE StartReceive;
 { Synchronisation mit Sender herstellen }
BEGIN
  OwnState := Closed;         { Timer abschalten ...            }
  Port[daten] := ACK1;        { besttigen...                   }
  swait(0);                   { Gegenbesttigung abwarten       }
  Port[daten] := ACK2;        { und wieder besttigen...        }
END;

PROCEDURE StopReceive;
BEGIN
  Port[daten] := NULL2;
  OwnState := Free;
END;

PROCEDURE SendBlock(VAR Buf; Size : WORD); ASSEMBLER;
  { Die eigentliche Kernroutine des Sendens. }
ASM
  PUSH DS
  CLD
@Restart:
  POP  DS
  PUSH DS
  MOV  CX,Size  { Gre laden               }
  Inc  CX       { bei ungerader Anzahl gerade laden }
  SHR  CX,1     { in Worte umrechnen        }
  MOV  AX,Size  { Lnge laden               }
  MOV  DX,daten { Adresse der Schnittstelle }
  MOV  BH,ACK1 XOR $80
  MOV  BL,ACK2 XOR $80
  XOR  DI,DI    { "Quersumme"               }
  LDS  SI,Buf   { Adresse des Puffers laden }
@Loop:
  LODSW         { zwei Byte auf einmal laden }
  XOR  DI,AX
  ROL  DI,1
@first:         { Einsprungadresse fr Lnge }
  OUT  DX,AL    { low-Byte raus }
  ADD  DX,2
  MOV  AL,DoStrobe { Strobe setzen }
  OUT  DX,AL
  Dec  DX       { Auf ACK warten }
@ACK1:
  IN   AL,DX    { ACK-Signal lesen }
  AND  AL,$F8
  CMP  AL,BH    { noch ACK1 aktiv (ACK2>=ACK1+8) }
  JNE  @ACK1
  Dec  DX       { High-Byte -> Daten }
  MOV  AL,AH
  OUT  DX,AL
  ADD  DX,2
  MOV  AL,NotStrobe { Strobe lschen -> Steuer }
  OUT  DX,AL
  Dec  DX
@ACK2:
  IN   AL,DX    { auf ACK1 warten }
  MOV  AH,AL
  AND  AL,$C8
  CMP  AL,BL
  JNE  @ACK2
  Dec  DX
  SUB  CX,1
  JB   @ProcEnd
  JNZ  @Loop
  MOV  AX,DI    { di bertragen }
  JMP  @first
@ProcEnd:
  AND  AH,$F8
  CMP  AH,ACKError XOR $80 { Fehler beim Schreiben ? }
  JNZ  @NoRestart
  JMP  @Restart
@NoRestart:
  POP  DS
END;

FUNCTION ReceiveBlock(VAR Buf; Size : WORD) : WORD;
VAR
  xl,xh : BYTE;
  good  : BOOLEAN;
BEGIN
 REPEAT
  { ---------- Transfer der eigentlichen Daten --------- }
  ASM
    MOV  AH,xh     { Funktionsergebnis sichern  }
    MOV  AL,xl
    MOV  AX,Size
    PUSH AX
    CLD
    MOV  CX,AX     { Gre laden               }
    Inc  CX        { bei ungerader Anzahl gerade Anzahl laden }
    SHR  CX,1
    MOV  DX,steuer { Adresse der Schnittstelle }
    MOV  BL,ACK1
    MOV  BH,ACK2
    XOR  SI,SI
    LES  DI,Buf    { Adresse des Puffers laden }
  @Loop:
  @Wait1:          { warten, bis strobe gesetzt... }
    IN   AL,DX
    SHR  AL,1
    JNC  @Wait1
    AND  AL,7
    MOV  AH,AL
    Dec  DX        { status holen... }
    IN   AL,DX
    AND  AL,$F8    { korrigieren... }
    OR   AL,AH
    XOR  AL,$85
    STOSB          { und abspeichern... }
    SUB  AH,AH
    XOR  SI,AX
    Dec  DX
    MOV  AL,BL     { ACK-Signal senden }
    OUT  DX,AL
    ADD  DX,2
  @Wait2:          { warten, bis strobe nicht gesetzt... }
    IN   AL,DX
    SHR  AL,1
    JC   @Wait2
    AND  AL,7
    MOV  AH,AL
    Dec  DX        { status holen... }
    IN   AL,DX
    AND  AL,$F8    { korrigieren... }
    OR   AL,AH
    XOR  AL,$85
    STOSB          { und abspeichern... }
    MOV  AH,AL
    SUB  AL,AL
    XOR  SI,AX
    ROL  SI,1
    Dec  DX
    MOV  AL,BH     { ACK-Not-Signal senden }
    OUT  DX,AL
    ADD  DX,2
    Dec  CX
    JNZ  @Loop
    POP  AX        { Funktionsergebnis }
    MOV  @Result,AX
    MOV  AX,SI
    MOV  xh,AH
    MOV  xl,AL
  END;
  swait(1);            { warten,                         }
  good := xl=empfang;  { lower Byte lesen                }
  Port[daten] := ACK1; { und Handshake                   }
  swait(0);            { warten,                         }
  good := good AND (xh=empfang); { higher Byte lesen     }
  IF good THEN
   Port[daten] := ACK2 { und Handshake                   }
  ELSE
   Port[daten] := ACKError;
 UNTIL good;
END;

END.

