(* ====================================================== *)
(*                      PARCRC.PAS                        *)
(*           Datenbertragung mit CRC-Checksumme          *)
(*         (c) 1993 Ralf Hensmann & DMV-Verlag            *)
(* ====================================================== *)
{$A+,B-,D+,E+,F-,G-,I+,L+,N-,O-,P-,Q-,R-,S+,T-,V+,X+,Y+}
{$M 16384,0,655360}

UNIT ParCRC;

INTERFACE

USES ParData, Crc, Crt;

  PROCEDURE SendCRCBuf(VAR Buf; Size : WORD);
  (* SendCRCBuf bertrgt einen Block mit anschlieendem  *)
  (* CRC-16-Wort. Ist die bertragung nicht geglckt,     *)
  (* wird wiederholt. Vor dem ersten Aufruf von           *)
  (* SendCRCBuf mu die CRC-Tabelle gesetzt sein.         *)

  PROCEDURE ReceiveCRCBuf(VAR Buf; Size : WORD);
  (* ReceiveCRCBuf empfngt einen Block mit anschlieendem*)
  (* CRC-Wert. Die Gre mu bekannt sein, da sonst keine *)
  (* fehlerfreie bertragung mglich ist.                 *)

IMPLEMENTATION

TYPE
  OS = RECORD            (* oder LongRec aus Unit Objects *)
         o, s : WORD;
       END;

  PROCEDURE SendCRCBuf(VAR Buf; Size : WORD);
  VAR
    Answer   : WORD;
    BufPtr   : POINTER;
    Buffer   : ARRAY [0..65530] OF BYTE ABSOLUTE Buf;
    i        : WORD;
  BEGIN
    (* Bei Fehler Abbruch *)
    IF LastResult <> ReadyToTransfer THEN Exit;
    (* in 512-Byte Blcke zerlegen, um bei Fehlern nicht  *)
    (* ganzen Block neu zu bertragen.                    *)
    i := 0;
    WHILE Size > 512 DO BEGIN
      SendCRCBuf(Buffer[i], 512);
      IF LastResult <> ReadyToTransfer THEN Exit;
      Inc(i, 512);  Dec(Size, 512);
    END;
    (* Adresse normalisieren *)
    BufPtr := @Buffer[i];
    OS(BufPtr).s := OS(BufPtr).s + OS(BufPtr).o SHR 4;
    OS(BufPtr).o := OS(BufPtr).o AND $F;
    (* CRC-Summe berechnen *)
    CRC_16 := 0;
    Build_crc(BufPtr^, Size);
    (* bertragen *)
    REPEAT
      IF TransferMode = Transfer4 THEN BEGIN
        (* 4-Bit-betragung *)
        SendBlock4(BufPtr^, Size);
        SendBlock4(CRC_16, 2);
        ReceiveBlock4(Answer, 2);
      END ELSE BEGIN
        (* 8-Bit-bertragung *)
        SendBlock(BufPtr^, Size);
        SendBlock(CRC_16, 2);
        ReceiveBlock(Answer, 2);
      END;
    UNTIL (Answer = 0) OR (LastResult <> ReadyToTransfer);
  END;

  PROCEDURE ReceiveCRCBuf(VAR Buf; Size : WORD);
  VAR
    Follow   : WORD;
    BufPtr   : POINTER;
    Buffer   : ARRAY [0..65530] OF BYTE ABSOLUTE Buf;
    i        : WORD;
  BEGIN
    IF LastResult <> ReadyToTransfer THEN Exit;
    (* Zerlegen wie oben *)
    i := 0;
    WHILE Size > 512 DO BEGIN
      ReceiveCRCBuf(Buffer[i], 512);
      IF LastResult <> ReadyToTransfer THEN Exit;
      Inc(i, 512);  Dec(Size, 512);
    END;
    (* Adresse normalisieren *)
    BufPtr := @Buffer[i];
    OS(BufPtr).s := OS(BufPtr).s + OS(BufPtr).o SHR 4;
    OS(BufPtr).o := OS(BufPtr).o AND $F;
    REPEAT
      CRC_16 := 0;
      IF TransferMode = Transfer4 THEN BEGIN
        (* 4-Bit-betragung *)
        ReceiveBlock4(BufPtr^, Size);
        Build_crc(BufPtr^, Size);
        ReceiveBlock4(Follow, 2);
        CRC_16 := CRC_16 XOR Follow;
        SendBlock4(CRC_16, 2);
      END ELSE BEGIN
        (* 8-Bit-betragung *)
        ReceiveBlock(BufPtr^, Size);
        Build_crc(BufPtr^, Size);
        ReceiveBlock(Follow, 2);
        CRC_16 := CRC_16 XOR Follow;
        SendBlock(CRC_16, 2);
      END;
    UNTIL (CRC_16 = 0) OR (LastResult <> ReadyToTransfer);
  END;

END.
(* ====================================================== *)
(*                  Ende von PARCRC.PAS                   *)
