{******************************************************}
{   CLIP.PAS                                           }
{   Pascal for Windows                                 }
{   originally a Demo program                          }
{   Copyright (c) 1991 by Borland International        }
{   Enhancements (c) 1992 by Reebear @ Borland Germany }
{   further enh. (c) 1993 by Reebear @ Home            }
{******************************************************}

PROGRAM Clip;

{$R CLIP.RES}

USES
  WinTypes, WinProcs, WinDos, OWindows, ODialogs, Strings, OStdDlgs, OMemory;

CONST
  cm_Load        = 100;
  cm_About       = 103;
  cm_CopyBClip   = 200;
  cm_PasteBClip  = 201;
  cm_EraseClient = 202;
  cm_CopyTClip   = 301;
  cm_PasteTClip  = 302;

TYPE
  TApp = OBJECT(tApplication)
    PROCEDURE InitMainWindow; VIRTUAL;
  END;

  PClipWindow = ^TClipWindow;
  TClipWindow = OBJECT(tWindow)
    BitMapHandle: hBitmap;
    IconizedBits: hBitmap;
    IconImageValid: BOOLEAN;
    Width, Height: LONGINT;
    EditWin: pEdit;
    CONSTRUCTOR Init(aParent: pWindowsObject; Title: pChar);
    DESTRUCTOR Done; VIRTUAL;
    PROCEDURE About(VAR Message: tMessage); VIRTUAL cm_First + cm_About;
    PROCEDURE GetBitmapData(VAR TheFile: FILE; BitsHandle: tHandle;
      BitsByteSize: LONGINT);
    PROCEDURE GetWindowClass(VAR WndClass: tWndClass); VIRTUAL;
    FUNCTION LoadBitmapFile(Name: pChar): BOOLEAN;
    PROCEDURE LoadImage(VAR Message: tMessage); VIRTUAL cm_First + cm_Load;
    FUNCTION OpenDIB(VAR TheFile: FILE): BOOLEAN;
    PROCEDURE Paint(PaintDC: hDC; VAR PaintInfo: tPaintStruct); VIRTUAL;
    PROCEDURE WMSize(VAR Message: tMessage); VIRTUAL wm_Size;

    (* neu : *)
    PROCEDURE CopyBClip (VAR Message: tMessage); VIRTUAL cm_First + cm_CopyBClip;
    PROCEDURE PasteBClip (VAR Message: tMessage); VIRTUAL cm_First + cm_PasteBClip;
    PROCEDURE EraseClient (VAR Message: tMessage); VIRTUAL cm_First + cm_EraseClient;
    PROCEDURE CopyTClip (VAR Message: tMessage); VIRTUAL cm_First + cm_CopyTClip;
    PROCEDURE PasteTClip (VAR Message: tMessage); VIRTUAL cm_First + cm_PasteTClip;
  END;

{ __ahIncr, ordinal 114, is a 'magic' function. Defining this
  function causes Windows to patch the value into the passed
  reference.  This makes it a type of global variable. To use
  the value of AHIncr, use Ofs(AHIncr). }
PROCEDURE AHIncr; FAR; EXTERNAL 'KERNEL' INDEX 114;

{ TClipWindow }

CONSTRUCTOR TClipWindow.Init(aParent: pWindowsObject; Title: pChar);
VAR
  DC: hDC;
BEGIN
  tWindow.Init(aParent, Title);
  BitMapHandle := 0;
  DC := GetDC(GetFocus);
  IconizedBits := CreateCompatibleBitmap(DC, 64, 64);
  ReleaseDC(GetFocus, DC);
  IconImageValid := FALSE;
END;

DESTRUCTOR TClipWindow.Done;
BEGIN
  IF BitMapHandle <> 0 THEN DeleteObject(BitMapHandle);
  DeleteObject(IconizedBits);
  tWindow.Done;
END;

PROCEDURE TClipWindow.About(VAR Message: tMessage);
VAR
  Dialog: tDialog;
BEGIN
  Dialog.Init(@Self, 'About');
  Dialog.Execute;
  Dialog.Done;
END;

{ Copys the bitmap bit data from the file into memory. Since
  copying cannot cross a segment (64K) boundary, we are forced
  to do segment arithmetic to compute the next segment.  Created
  a LongType type to simplify the process. }
PROCEDURE TClipWindow.GetBitmapData(VAR TheFile: FILE;
  BitsHandle: tHandle; BitsByteSize: LONGINT);
TYPE
  LongType = RECORD
    CASE WORD OF
      0: (Ptr: POINTER);
      1: (Long: LONGINT);
      2: (Lo: WORD;
	  Hi: WORD);
  END;
VAR
  Count: LONGINT;
  Start, ToAddr, Bits: LongType;
BEGIN
  Start.Long := 0;
  Bits.Ptr := GlobalLock(BitsHandle);
  Count := BitsByteSize - Start.Long;
  WHILE Count > 0 DO
  BEGIN
    ToAddr.Hi := Bits.Hi + (Start.Hi * Ofs(AHIncr));
    ToAddr.Lo := Start.Lo;
    IF Count > $4000 THEN Count := $4000;
    BlockRead(TheFile, ToAddr.Ptr^, Count);
    Start.Long := Start.Long + Count;
    Count := BitsByteSize - Start.Long;
  END;
  GlobalUnlock(BitsHandle);
END;

PROCEDURE TClipWindow.GetWindowClass(VAR WndClass: tWndClass);
BEGIN
  tWindow.GetWindowClass(WndClass);

 { With a 0 as hIcon the program can write to the Icon in the paint method }
  WndClass.hIcon := 0;
  WndClass.lpszMenuName := 'Menu';
END;

{ Test if the passed file is a Windows 3.0 DI bitmap and if so read it.
  Report errors if unable to do so. Adjust the Scroller to the new
  bitmap dimensions. }
FUNCTION TClipWindow.LoadBitmapFile(Name: pChar): BOOLEAN;
VAR
  TheFile: FILE;
  TestWin30Bitmap: LONGINT;
  MemDC: hDC;
BEGIN
  LoadBitmapFile := FALSE;
  Assign(TheFile, Name);
  Reset(TheFile, 1);
  Seek(TheFile, 14);
  BlockRead(TheFile, TestWin30Bitmap, SizeOf(TestWin30Bitmap));
  IF TestWin30Bitmap = 40 THEN
    IF OpenDIB(TheFile) THEN
    BEGIN
      LoadBitmapFile := TRUE;
      IconImageValid := FALSE;
    END
    ELSE
      MessageBox(hWindow, 'Unable to create Windows 3.0 bitmap from file.',
	Name, mb_OK)
  ELSE
      MessageBox(hWindow, 'Not a Windows 3.0 bitmap file.  Convert using Paintbrush.', Name, mb_OK);
  Close(TheFile);
END;

PROCEDURE TClipWindow.LoadImage(VAR Message: tMessage);
VAR
  FileName: ARRAY[0..200] OF CHAR;
  CaptionBuffer: ARRAY [0..200] OF CHAR;
BEGIN
  IF Application^.ExecDialog(New(pFileDialog,
    Init(@Self, pChar(sd_FileOpen),
    StrCopy(FileName, '*.bmp')))) = id_OK THEN
    LoadBitmapFile(FileName);
  InvalidateRect(hWindow, NIL, FALSE);
END;

{ Attempt to open a Windows 3.0 device independent bitmap. }
FUNCTION TClipWindow.OpenDIB(VAR TheFile: FILE): BOOLEAN;
VAR
  bitCount: WORD;
  Size: WORD;
  longWidth: LONGINT;
  DCHandle: hDC;
  BitsPtr: POINTER;
  BitmapInfo: pBitmapInfo;
  BitsHandle, NewBitmapHandle: tHandle;
  NewPixelWidth, NewPixelHeight: WORD;
BEGIN
  OpenDIB := TRUE;
  Seek(TheFile, 28);
  BlockRead(TheFile, bitCount, SizeOf(bitCount));
  IF bitCount <= 8 THEN
  BEGIN
    Size := SizeOf(tBitmapInfoHeader) + ((1 SHL bitCount) * SizeOf(tRGBQuad));
    BitmapInfo := MemAlloc(Size);
    Seek(TheFile, SizeOf(tBitmapFileHeader));
    BlockRead(TheFile, BitmapInfo^, Size);
    NewPixelWidth := BitmapInfo^.bmiHeader.biWidth;
    NewPixelHeight := BitmapInfo^.bmiHeader.biHeight;
    longWidth := (((NewPixelWidth * bitCount) + 31) DIV 32) * 4;
    BitmapInfo^.bmiHeader.biSizeImage := longWidth * NewPixelHeight;
    GlobalCompact(-1);
    BitsHandle := GlobalAlloc(gMem_Moveable OR gMem_ZeroInit,
      BitmapInfo^.bmiHeader.biSizeImage);
    GetBitmapData(TheFile, BitsHandle, BitmapInfo^.bmiHeader.biSizeImage);
    DCHandle := CreateDC('Display', NIL, NIL, NIL);
    BitsPtr := GlobalLock(BitsHandle);
    NewBitmapHandle :=
      CreateDIBitmap(DCHandle, BitmapInfo^.bmiHeader, cbm_Init, BitsPtr,
      BitmapInfo^, 0);
    DeleteDC(DCHandle);
    GlobalUnlock(BitsHandle);
    GlobalFree(BitsHandle);
    FreeMem(BitmapInfo, Size);
    IF NewBitmapHandle <> 0 THEN
    BEGIN
      IF BitMapHandle <> 0 THEN DeleteObject(BitMapHandle);
      BitMapHandle := NewBitmapHandle;
      Width := NewPixelWidth;
      Height := NewPixelHeight;
    END
    ELSE
      OpenDIB := FALSE;
  END
  ELSE
    OpenDIB := FALSE;
END;

PROCEDURE TClipWindow.Paint(PaintDC: hDC; VAR PaintInfo: tPaintStruct);
VAR
  MemDC: hDC;
  OldBitmap: hBitmap;
  R: tRect;
BEGIN
  IF BitMapHandle <> 0 THEN
  BEGIN
    MemDC := CreateCompatibleDC(PaintDC);
    IF IsIconic(hWindow) AND IconImageValid THEN
    BEGIN
      OldBitmap := SelectObject(MemDC, IconizedBits);
      BitBlt(PaintDC, 0, 0, Width, Height, MemDC, 0, 0, SrcCopy);
    END
    ELSE
    BEGIN
      SelectObject(MemDC, BitMapHandle);
      BitBlt(PaintDC, 0, 0, Width, Height, MemDC, 0, 0, SrcCopy);
    END;
    DeleteDC(MemDC);
  END;
END;

PROCEDURE TClipWindow.WMSize(VAR Message: tMessage);
VAR
  DC, MemDC1, MemDC2: hDC;
  OldBitmap1, OldBitmap2: hBitmap;
  OldCursor: hCursor;
BEGIN
  IF NOT IconImageValid AND (Message.wParam = sizeIconic) AND
    (BitMapHandle <> 0) THEN
  BEGIN
    DC := GetDC(hWindow);
    MemDC1 := CreateCompatibleDC(DC);
    MemDC2 := CreateCompatibleDC(DC);
    ReleaseDC(hWindow, DC);
    OldBitmap1 := SelectObject(MemDC1, IconizedBits);
    OldBitmap2 := SelectObject(MemDC2, BitMapHandle);
    OldCursor := SetCursor(LoadCursor(0, idc_Wait));
    StretchBlt(MemDC1, 0, 0, Message.lParamLo, Message.lParamHi, MemDC2,
      0, 0, Width, Height, SrcCopy);
    SetCursor(OldCursor);
    SelectObject(MemDC1, OldBitmap1);
    SelectObject(MemDC2, OldBitmap2);
    DeleteDC(MemDC1);
    DeleteDC(MemDC2);
    IconImageValid := TRUE;
  END;
  InvalidateRect(hWindow, NIL, FALSE);
END;

(* neu : *)

PROCEDURE TClipWindow.CopyBClip (VAR Message: tMessage);
VAR
  DevCon, MemCon: hDC;
  Bitmap: hBitmap;
  Rect: tRect;
BEGIN
  DevCon := GetDC (hWindow);
  MemCon := CreateCompatibleDC (DevCon);
  GetClientRect (hWindow, Rect);
  Bitmap := CreateCompatibleBitmap (DevCon, Rect.Right, Rect.Bottom);
  IF (Bitmap <> 0) THEN BEGIN
    SelectObject (MemCon, Bitmap);
    StretchBlt (MemCon, 0, 0, Rect.Right, Rect.Bottom,
                DevCon, Rect.Left, Rect.Top, Rect.Right, Rect.Bottom,
                SrcCopy);
    OpenClipboard (hWindow);
    EmptyClipboard;
    SetClipboardData (cf_Bitmap, Bitmap);
    CloseClipboard;
    InvalidateRect (hWindow, NIL ,TRUE);
  END
  ELSE MessageBeep (0);
  DeleteDC (MemCon);
  ReleaseDC (hWindow, DevCon);
END;

PROCEDURE TClipWindow.PasteBClip (VAR Message: tMessage);
VAR
  DevCon, ClipDC: hDC;
  ClipBmp: hBitmap;
  BM: tBitmap;
BEGIN
  IF OpenClipboard (hWindow) THEN
  BEGIN
    DevCon := GetDC (hWindow);
    ClipDC := CreateCompatibleDC (DevCon);
    IF IsClipboardFormatAvailable (cf_Bitmap) THEN
    BEGIN
      ClipBmp := GetClipboardData (cf_Bitmap);
      IF ((ClipBmp <> 0) AND
      (GetObject(ClipBmp, SizeOf(tBitmap), @BM) <> 0)) THEN BEGIN
        SelectObject (ClipDC, ClipBmp);
        BitBlt (DevCon, 0, 0, BM.bmWidth, BM.bmHeight, ClipDC, 0, 0, SrcCopy);
      END;
      Width := BM.bmWidth;
      Height := BM.bmHeight;
      CloseClipboard;
      BitMapHandle := ClipBmp;
      InvalidateRect (hWindow, NIL, TRUE);
    END;
    DeleteDC (ClipDC);
    ReleaseDC (hWindow, DevCon);
  END;
END;

PROCEDURE TClipWindow.EraseClient (VAR Message: tMessage);
BEGIN
  InvalidateRect (hWindow, NIL, TRUE);
  IF (BitMapHandle <> 0) THEN DeleteObject (BitMapHandle);
END;

PROCEDURE TClipWindow.CopyTClip (VAR Message: tMessage);
BEGIN
  EditWin := New (pEdit, Init(@Self, 200, NIL, 0, 0, 0, 0, 0, TRUE));
END;

PROCEDURE TClipWindow.PasteTClip (VAR Message: tMessage);
BEGIN
END;

{ TApp }

PROCEDURE TApp.InitMainWindow;
BEGIN
  MainWindow := New(PClipWindow, Init(NIL, 'Klipp-Klapp'));
END;

VAR
  App: TApp;
BEGIN
  App.Init('Stretch');
  App.Run;
  App.Done;
END.
